import React, { PropsWithChildren, useEffect, useState } from "react";
import { OIDC_LOGIN_URL } from "../../utils/baseUrl";
import { finnMiljø, Miljø } from "../../utils/miljø";

export enum AuthenticationStatus {
    Initialisert,
    Henter,
    Autentisert,
    LoggerUt,
    Feilet,
}

interface AuthenticationContextValue {
    status: AuthenticationStatus;
    setStatus: (status: AuthenticationStatus) => void;
}

export const AuthenticationContext = React.createContext<AuthenticationContextValue>({
    status: AuthenticationStatus.Initialisert,
    setStatus: () => {},
});

export const AuthenticationProvider: React.FC<PropsWithChildren<{}>> = ({ children }) => {
    const [status, setStatus] = useState<AuthenticationStatus>(AuthenticationStatus.Initialisert);

    useEffect(() => {
        if (finnMiljø() === Miljø.lokal) {
            setStatus(AuthenticationStatus.Autentisert);
        } else if (status === AuthenticationStatus.Initialisert) {
            setStatus(AuthenticationStatus.Henter);
            fetch(`${OIDC_LOGIN_URL}/employeeSSO/status`, {
                credentials: "include",
            })
                .then((res) => res.json())
                .then((json) => {
                    if (json.loggedIn) {
                        setStatus(AuthenticationStatus.Autentisert);
                        createCookieRefreshInterval();
                    } else {
                        setStatus(AuthenticationStatus.Henter);
                        redirectToLogin();
                    }
                });
        } else if (status === AuthenticationStatus.LoggerUt) {
            logoutAndRedirect();
        }
    }, [status]);

    return (
        <AuthenticationContext.Provider
            value={{
                status,
                setStatus,
            }}
        >
            {children}
        </AuthenticationContext.Provider>
    );
};

const createCookieRefreshInterval = () => {
    const MINUTTER_15 = 1000 * 60 * 15;

    const refreshCookie = () => {
        fetch(`${OIDC_LOGIN_URL}/employeeSSO/extend`, {
            credentials: "include",
        }).then((res) => {
            if (res.status === 401) {
                redirectToLogin();
            }
        });
    };

    refreshCookie();
    setInterval(refreshCookie, MINUTTER_15);
};

const redirectToLogin = () => {
    if (typeof window !== "undefined") {
        window.location.replace(
            `${OIDC_LOGIN_URL}/employeeSSO/login?returnUrl=${document.location.href}`
        );
    }
};

const logoutAndRedirect = () => {
    if (typeof window !== "undefined") {
        window.location.replace(
            `${OIDC_LOGIN_URL}/employeeSSO/logout?returnUrl=https://www.vegvesen.no/`
        );
    }
};
