import React, { PropsWithChildren, useEffect, useState } from "react";
import dayjs, { Dayjs } from "dayjs";
import { ApiRespons } from "../api";
import { KontraktBrukerDto, useHentKontraktinfo } from "../useHentKontraktinfo";
import { tilKontrakter } from "./kartdataContextHelpers";
import { finnInitAktivKontrakt, finnInitTidspunkt } from "../../kartSide/utils/kartUrlUtils";

export interface KartContextValue {
    tidspunkt: TidspunktValue;
    kontrakt: KontraktValue;
    brukerInfo?: ApiRespons<KontraktBrukerDto>;
    intro: IntroValue;
}

export interface TidspunktValue {
    verdi: Dayjs;
    oppdater: (tid: Dayjs) => void;
}

interface KontraktValue {
    finnes: boolean;
    alle?: Kontrakt[];
    aktiv?: Kontrakt;
    oppdaterAktiv: (kontrakt: Kontrakt) => void;
}

export interface Kontrakt {
    navn: string;
    navnNvdb: string;
    elrappId: number;
}

interface IntroValue {
    vis: boolean;
    oppdater: (vis: boolean) => void;
    toggle: () => void;
}

export const KartdataContext = React.createContext<KartContextValue>({
    tidspunkt: {
        verdi: dayjs(),
        oppdater: (_) => {},
    },
    kontrakt: {
        finnes: false,
        alle: undefined,
        aktiv: undefined,
        oppdaterAktiv: (_) => {},
    },
    intro: {
        vis: false,
        oppdater: (_) => {},
        toggle: () => {},
    },
});

export const KartdataProvider: React.FC<PropsWithChildren<{}>> = ({ children }) => {
    const [tidspunkt, setTidspunkt] = useState<Dayjs>(finnInitTidspunkt());
    const [aktivKontrakt, setAktivKontrakt] = useState<Kontrakt | undefined>();
    const [kontrakter, setKontrakter] = useState<Kontrakt[] | undefined>();
    const [visIntro, setVisIntro] = useState<boolean>(false);

    const brukerInfo = useHentKontraktinfo();

    useEffect(() => {
        if (brukerInfo && brukerInfo.suksess) {
            setKontrakter(tilKontrakter(brukerInfo.data.kontrakter ?? []));
        }
    }, [brukerInfo]);

    useEffect(() => {
        const førsteKontrakt = finnInitAktivKontrakt(kontrakter);
        førsteKontrakt && setAktivKontrakt(førsteKontrakt);
    }, [kontrakter]);

    return (
        <KartdataContext.Provider
            value={{
                tidspunkt: {
                    verdi: tidspunkt,
                    oppdater: (tid: Dayjs) => setTidspunkt(tid),
                },
                kontrakt: {
                    finnes: kontrakter !== undefined && aktivKontrakt !== undefined,
                    alle: kontrakter,
                    aktiv: aktivKontrakt,
                    oppdaterAktiv: (kontrakt: Kontrakt) => setAktivKontrakt({ ...kontrakt }),
                },
                brukerInfo: brukerInfo,
                intro: {
                    vis: visIntro,
                    oppdater: (vis) => setVisIntro(vis),
                    toggle: () => setVisIntro(!visIntro),
                },
            }}
        >
            {children}
        </KartdataContext.Provider>
    );
};
