import React, { PropsWithChildren, useContext, useState } from "react";
import { ApiRespons, fetcherPost } from "../api";
import { GCP_MEDIATOR_URL } from "../../utils/baseUrl";
import { Produksjonsdetaljer } from "../../kartSide/types/produksjonsdetaljer";
import { Materiale, MaterialKategori, MaterialType } from "../../kartSide/types/materiale";
import { tilEnkelTimestampStr } from "../../utils/tid";
import { Dayjs } from "dayjs";
import { MaterialOgAktivitetsNavn } from "./ProduksjonsfilterContext";
import { Point } from "geojson";

export interface ProduksjonsdetaljerDto {
    produksjonsdataRestRec: number[];
    kjoeretoyid?: string;
    datoLogget: string;
    hastighetkmhmax: number;
    hastighetkmhavg: number;
    erplognede?: boolean;
    ertoerrstoffsprederpaa?: boolean;
    ervaatstoffsprederpaa?: boolean;
    sprederbredde?: number;
    dosering?: number;
    erhoevelskjaernede?: boolean;
    erunderliggendeskjaernede?: boolean;
    ersideplogibruk?: boolean;
    materiale?: string;
    eksternReferanse?: string;
    minutterSiden: number;
    startLokasjon: LokasjonOgRecordId;
    sluttLokasjon: LokasjonOgRecordId;
    vegsystemreferansekortform?: string;
    rodeId: string[];
}

interface LokasjonOgRecordId {
    recordId: number;
    punkt: Point;
}

interface ProduksjonsdetaljerCache {
    [id: string]: ApiRespons<Produksjonsdetaljer[]>;
}

interface CacheContextValue {
    pushToCache: (key: string, verdi: ApiRespons<Produksjonsdetaljer[]>) => void;

    hentProduksjonsdetaljer: (
        målepunkter: number[],
        tidspunkt: Dayjs
    ) => Promise<ApiRespons<Produksjonsdetaljer[]>> | undefined;

    tømCache: () => void;
}

export const ProduksjonsdetaljerCacheContext = React.createContext<CacheContextValue>({
    pushToCache: (key: string, verdi: ApiRespons<Produksjonsdetaljer[]>) => {},
    hentProduksjonsdetaljer: (målepunkter, tidspunkt) => undefined,
    tømCache: () => {},
});

export const useProduksjonsdetaljerCache = () => useContext(ProduksjonsdetaljerCacheContext);

export const ProduksjonsdetaljerCacheProvider: React.FC<PropsWithChildren<{}>> = ({ children }) => {
    const [cache, setCache] = useState<ProduksjonsdetaljerCache>({});

    const pushToCache = (key: string, verdi: ApiRespons<Produksjonsdetaljer[]>) => {
        cache[key] = verdi;
        setCache(cache);
    };

    const hentData = async (
        målepunkter: number[],
        tidspunkt: Dayjs
    ): Promise<ApiRespons<Produksjonsdetaljer[]>> => {
        const url = `${GCP_MEDIATOR_URL}/aggregerte-produksjonsdetaljer?tidspunkt=${tilEnkelTimestampStr(
            tidspunkt
        )}`;
        const body = { målepunkter: målepunkter };

        if (cache[målepunkter.toString()]) {
            return cache[målepunkter.toString()];
        }
        return fetcherPost<ProduksjonsdetaljerDto[]>(url, JSON.stringify(body)).then((response) =>
            response.suksess
                ? {
                      ...response,
                      data: response.data.map((dto) => dtoTilProduksjonsdetaljer(dto)),
                  }
                : response
        );
    };

    const tømCache = () => {
        setCache({});
    };

    return (
        <ProduksjonsdetaljerCacheContext.Provider
            value={{
                pushToCache: pushToCache,
                hentProduksjonsdetaljer: hentData,
                tømCache: tømCache,
            }}
        >
            {children}
        </ProduksjonsdetaljerCacheContext.Provider>
    );
};

export const dtoTilProduksjonsdetaljer = (dto: ProduksjonsdetaljerDto): Produksjonsdetaljer => ({
    produksjonsdataRestRec: dto.produksjonsdataRestRec,
    kjøretøyId: dto.kjoeretoyid,
    datoLogget: dto.datoLogget,
    hastighetKmHMax: dto.hastighetkmhmax,
    hastighetKmHAvg: dto.hastighetkmhavg,
    materiale: dtoTilMateriale(dto),
    erPlogNede: dto.erplognede ?? false,
    erTørrstoffsprederPå: dto.ertoerrstoffsprederpaa,
    erVåtstoffsprederPå: dto.ervaatstoffsprederpaa,
    eksternReferanse: dto.eksternReferanse,
    erUnderliggendeSkjærNede: dto.erunderliggendeskjaernede ?? false,
    erHøvelskjærNede: dto.erhoevelskjaernede ?? false,
    erSideplogIBruk: dto.ersideplogibruk ?? false,
    dosering: dto.dosering,
    sprederbredde: dto.sprederbredde,
    minutterSiden: dto.minutterSiden,
    startLokasjon: dto.startLokasjon,
    sluttLokasjon: dto.sluttLokasjon,
    vegsystemreferansekortform: dto.vegsystemreferansekortform,
    rodeId: dto.rodeId,
});

const dtoTilMateriale = (dto: ProduksjonsdetaljerDto): Materiale | undefined => {
    if (dto.materiale === "Tørrsand") {
        return {
            navn: MaterialOgAktivitetsNavn.Tørrsand,
            type: MaterialType.Tørrsand,
            kategori: MaterialKategori.Sand,
        };
    }
    if (dto.materiale === "Sand med saltinnblanding") {
        return {
            navn: MaterialOgAktivitetsNavn.SandMedSaltInnblanding,
            type: MaterialType.SandMedSaltInnblanding,
            kategori: MaterialKategori.Sand,
        };
    }
    if (dto.materiale === "Fastsand") {
        return {
            navn: MaterialOgAktivitetsNavn.Fastsand,
            type: MaterialType.Fastsand,
            kategori: MaterialKategori.Sand,
        };
    }
    if (dto.materiale === "Tørrsalt (NaCl)") {
        return {
            navn: MaterialOgAktivitetsNavn.Tørrsalt,
            type: MaterialType.Tørrsalt,
            kategori: MaterialKategori.Salt,
        };
    }
    if (dto.materiale === "Befuktet salt (NaCl)") {
        return {
            navn: MaterialOgAktivitetsNavn.BefuktetSalt,
            type: MaterialType.BefuktetSalt,
            kategori: MaterialKategori.Salt,
        };
    }
    if (dto.materiale === "Saltslurry (NaCl)" || dto.materiale === "Saltslurry (materialKode 11)") {
        return {
            navn: MaterialOgAktivitetsNavn.BefuktetFinkornetSalt,
            type: MaterialType.BefuktetFinkornetSalt,
            kategori: MaterialKategori.Salt,
        };
    }
    if (dto.materiale === "Saltløsning (NaCl)") {
        return {
            navn: MaterialOgAktivitetsNavn.Saltløsning,
            type: MaterialType.Saltløsning,
            kategori: MaterialKategori.Salt,
        };
    }
    if (dto.materiale === "Magnesiumklorid (MgCl)") {
        return {
            navn: MaterialOgAktivitetsNavn.Magnesiumklorid,
            type: MaterialType.Magnesiumklorid,
            kategori: MaterialKategori.Salt,
        };
    }
};
