import React, { PropsWithChildren, useContext, useState } from "react";
import { ApiRespons, fetcher } from "../api";
import { Produksjonslenke } from "../../kartSide/types/produksjonslenke";
import { Dayjs } from "dayjs";
import { GCP_MEDIATOR_URL } from "../../utils/baseUrl";
import { tilEnkelTimestampStr } from "../../utils/tid";
import { dtoTilProduksjonslenke, ProduksjonslenkeDto } from "../useHentProduksjonslenker";

export interface Øktfilter {
    verdi: string;
    valgt: boolean;
}
interface ØktCache {
    [id: string]: ApiRespons<Produksjonslenke[]>;
}
interface ØktfilterContextValue {
    øktfilter: Øktfilter;
    oppdaterFilter: (verdi: Øktfilter) => void;
    pushToØktCache: (key: string, verdi: ApiRespons<Produksjonslenke[]>) => void;
    hentØkt: (økt: string, tidspunkt: Dayjs) => Promise<ApiRespons<Produksjonslenke[]>> | undefined;
    tømCache: () => void;
    laster: boolean;
    gjeldendeØkt: Produksjonslenke[];
}

const defaultValue = {
    øktfilter: {
        verdi: "",
        valgt: false,
    },
    oppdaterFilter: (verdi: Øktfilter) => {},
    pushToØktCache: (key: string, verdi: ApiRespons<Produksjonslenke[]>) => {},
    hentØkt: (økt: string, tidspunkt: Dayjs) => undefined,
    tømCache: () => {},
    laster: false,
    gjeldendeØkt: [],
};

export const ØktfilterContext = React.createContext<ØktfilterContextValue>(defaultValue);

export const useOktContext = () => useContext(ØktfilterContext);

export const ØktfilterProvider: React.FC<PropsWithChildren<{}>> = ({ children }) => {
    const [filter, setFilter] = useState<Øktfilter>(defaultValue.øktfilter);
    const [cache, setCache] = useState<ØktCache>({});
    const [laster, setLaster] = useState<boolean>(false);
    const [gjeldendeØkt, setGjeldendeØkt] = useState<Produksjonslenke[]>([]);

    const pushToCache = (key: string, verdi: ApiRespons<Produksjonslenke[]>) => {
        cache[key] = verdi;
        setCache(cache);
    };

    const hentData = async (
        økt: string,
        tidspunkt: Dayjs
    ): Promise<ApiRespons<Produksjonslenke[]>> => {
        setLaster(true);

        if (cache[(økt + tidspunkt).toString()]) {
            setLaster(false);
            return cache[(økt + tidspunkt).toString()];
        }
        return fetcher<ProduksjonslenkeDto[]>(
            `${GCP_MEDIATOR_URL}/økt?økt=${økt}&tidspunkt=${tilEnkelTimestampStr(tidspunkt)}`
        ).then((response) => {
            setLaster(false);
            if (response.suksess) {
                const produksjonslenker = response.data.map((lenke) =>
                    dtoTilProduksjonslenke(lenke)
                );
                setGjeldendeØkt(produksjonslenker);
                pushToCache((økt + tidspunkt).toString(), {
                    ...response,
                    data: produksjonslenker,
                });
                return {
                    ...response,
                    data: produksjonslenker,
                };
            }
            return response;
        });
    };

    const tømCache = () => {
        setCache({});
    };

    return (
        <ØktfilterContext.Provider
            value={{
                øktfilter: filter,
                oppdaterFilter: setFilter,
                pushToØktCache: pushToCache,
                hentØkt: hentData,
                tømCache: tømCache,
                laster: laster,
                gjeldendeØkt: gjeldendeØkt,
            }}
        >
            {children}
        </ØktfilterContext.Provider>
    );
};
