import useSWR from "swr";
import { ApiRespons, fetcher, ikkeRevalidate } from "./api";
import { GCP_MEDIATOR_URL } from "../utils/baseUrl";
import { Geometry } from "geojson";
import { Dayjs } from "dayjs";
import { tilEnkelTimestampStr } from "../utils/tid";
import { Produksjonslenke } from "../kartSide/types/produksjonslenke";
import { Produksjonstype } from "../kartSide/types/produksjonstype";
import { Materiale, MaterialKategori, MaterialType } from "../kartSide/types/materiale";
import { Aktivitet, AktivitetKategori, AktivitetType } from "../kartSide/types/aktivitet";
import { MaterialOgAktivitetsNavn } from "./context/ProduksjonsfilterContext";

export interface ProduksjonslenkeDto {
    veglenkesekvensId: number;
    veglenkeNummer: number;
    økt: string;
    målepunkter: number[];
    geometri: Geometry;
    minutterSiden: number;
    produksjonstyper: ProduksjonstypeDto[];
    produksjonsmateriale?: ProduksjonsmaterialeDto;
    hastighetKmH: number;
}

interface ProduksjonstypeDto {
    aktivitet: ProduksjonsaktivitetDto;
    type: AktivitetTypeDto;
    ibruk: boolean;
}

enum ProduksjonsaktivitetDto {
    PlogNede = "PlogNede",
    Sideplog = "Sideplog",
    UnderliggendeSkjær = "UnderliggendeSkjær",
    TungHøvel = "TungHøvel",
}

enum AktivitetTypeDto {
    Brøyting = "Brøyting",
    Høvling = "Høvling",
}

interface ProduksjonsmaterialeDto {
    materiale: MaterialeDto;
    type: MaterialTypeDto;
}

enum MaterialeDto {
    Tørrsand = "Tørrsand",
    Fastsand = "Fastsand",
    SandMedSaltInnblanding = "SandMedSaltInnblanding",
    Tørrsalt = "Tørrsalt",
    BefuktetSalt = "BefuktetSalt",
    BefuktetFinkornetSalt = "BefuktetFinkornetSalt",
    Saltløsning = "Saltløsning",
    Magnesiumklorid = "Magnesiumklorid",
}

enum MaterialTypeDto {
    Sand = "Sand",
    Salt = "Salt",
}

export function useHentProduksjonslenker(
    kontraktnavn: string | undefined,
    tidspunkt: Dayjs
): ApiRespons<Produksjonslenke[]> | undefined {
    const { data } = useSWR<ApiRespons<ProduksjonslenkeDto[]>>(
        kontraktnavn
            ? `${GCP_MEDIATOR_URL}/siste_produksjon_v3?kontraktnavn=${kontraktnavn}&tidspunkt=${tilEnkelTimestampStr(
                  tidspunkt
              )}`
            : null,
        fetcher,
        ikkeRevalidate
    );

    if (data?.suksess) {
        return {
            ...data,
            data: data.data.map((lenke) => dtoTilProduksjonslenke(lenke)),
        };
    }
    return data;
}

export const dtoTilMateriale = (dto: ProduksjonsmaterialeDto): Materiale => {
    switch (dto.materiale) {
        case MaterialeDto.Tørrsand:
            return {
                navn: MaterialOgAktivitetsNavn.Tørrsand,
                type: MaterialType.Tørrsand,
                kategori: MaterialKategori.Sand,
            };
        case MaterialeDto.Fastsand:
            return {
                navn: MaterialOgAktivitetsNavn.Fastsand,
                type: MaterialType.Fastsand,
                kategori: MaterialKategori.Sand,
            };
        case MaterialeDto.SandMedSaltInnblanding:
            return {
                navn: MaterialOgAktivitetsNavn.SandMedSaltInnblanding,
                type: MaterialType.SandMedSaltInnblanding,
                kategori: MaterialKategori.Sand,
            };
        case MaterialeDto.Tørrsalt:
            return {
                navn: MaterialOgAktivitetsNavn.Tørrsalt,
                type: MaterialType.Tørrsalt,
                kategori: MaterialKategori.Salt,
            };
        case MaterialeDto.BefuktetSalt:
            return {
                navn: MaterialOgAktivitetsNavn.BefuktetSalt,
                type: MaterialType.BefuktetSalt,
                kategori: MaterialKategori.Salt,
            };
        case MaterialeDto.BefuktetFinkornetSalt:
            return {
                navn: MaterialOgAktivitetsNavn.BefuktetFinkornetSalt,
                type: MaterialType.BefuktetFinkornetSalt,
                kategori: MaterialKategori.Salt,
            };
        case MaterialeDto.Saltløsning:
            return {
                navn: MaterialOgAktivitetsNavn.Saltløsning,
                type: MaterialType.Saltløsning,
                kategori: MaterialKategori.Salt,
            };
        case MaterialeDto.Magnesiumklorid:
            return {
                navn: MaterialOgAktivitetsNavn.Magnesiumklorid,
                type: MaterialType.Magnesiumklorid,
                kategori: MaterialKategori.Salt,
            };
    }
};

export const dtoTilProduksjonstype = (dto: ProduksjonstypeDto): Produksjonstype => ({
    aktivitet: dtoTilAktivitet(dto.aktivitet),
    ibruk: dto.ibruk,
});

const dtoTilAktivitet = (dto: ProduksjonsaktivitetDto): Aktivitet => {
    switch (dto) {
        case ProduksjonsaktivitetDto.PlogNede:
            return {
                navn: MaterialOgAktivitetsNavn.PlogNede,
                type: AktivitetType.PlogNede,
                kategori: AktivitetKategori.Brøyting,
            };
        case ProduksjonsaktivitetDto.Sideplog:
            return {
                navn: MaterialOgAktivitetsNavn.Sideplog,
                type: AktivitetType.Sideplog,
                kategori: AktivitetKategori.Brøyting,
            };
        case ProduksjonsaktivitetDto.UnderliggendeSkjær:
            return {
                navn: MaterialOgAktivitetsNavn.UnderliggendeSkjær,
                type: AktivitetType.UnderliggendeSkjær,
                kategori: AktivitetKategori.Høvling,
            };
        case ProduksjonsaktivitetDto.TungHøvel:
            return {
                navn: MaterialOgAktivitetsNavn.TungHøvel,
                type: AktivitetType.TungHøvel,
                kategori: AktivitetKategori.Høvling,
            };
    }
};
export const dtoTilProduksjonslenke = (dto: ProduksjonslenkeDto): Produksjonslenke => ({
    veglenkesekvensId: dto.veglenkesekvensId,
    veglenkeNummer: dto.veglenkeNummer,
    økt: dto.økt,
    målepunkter: dto.målepunkter,
    geometri: dto.geometri,
    minutterSiden: dto.minutterSiden,
    produksjonstyper: dto.produksjonstyper.map((type) => dtoTilProduksjonstype(type)),
    produksjonsmateriale: dto.produksjonsmateriale
        ? dtoTilMateriale(dto.produksjonsmateriale)
        : undefined,
    hastighetKmH: dto.hastighetKmH,
});
