import styled from "styled-components";
import { Tekst } from "../../components/fellesKomponenter/typografi/Tekst";
import React, { useRef } from "react";
import { Checkbox } from "../../components/fellesKomponenter/checkboxer/Checkbox";
import { Spinner } from "../../components/fellesKomponenter/spinner/Spinner";
import { ApiRespons } from "../../api/api";
import { TrafikkmeldingerDto } from "../../api/useHentTrafikkmeldinger";
import { useDisableScrollAndClick } from "../../hooks/useDisableScrollAndClick";
import { useCloseOnOutsideClick } from "../../hooks/useCloseOnOutsideClick";
import localStorageService from "../../utils/storage/localstorageService";
import { LocalStorageKey } from "../../utils/storage/nøkler";
import { setLocalstorageDataMedGyldigDatoType } from "../../utils/storage/typer";
import { motion } from "framer-motion";

interface Props {
    kartvalg: KartvisningsValgProps;
    onClose: () => void;
    åpen: boolean;
}

export interface KartvisningsValgProps {
    visKameraer: boolean;
    setVisKameraer: (visKamera: boolean) => void;
    visBrøytebiler: boolean;
    setVisBrøytebiler: (visBrøytebiler: boolean) => void;
    visTrafikkmeldinger: boolean;
    handleTrafikkmeldingToggle: (visStengteVeier: boolean) => void;
    trafikkmeldinger: ApiRespons<TrafikkmeldingerDto[]> | undefined;
    visLegend?: boolean;
    settVisLegend?: (visLegend: boolean) => void;
    visDarkmode: boolean;
    settVisDarkmode: (visDarkmode: boolean) => void;
}

export const KartlagValg = ({ kartvalg, onClose, åpen }: Props) => {
    const ref = useRef(null);
    const id = "karlagvalg";
    useDisableScrollAndClick(ref);
    useCloseOnOutsideClick(id, onClose);

    const handleKameraerToggle = () => {
        kartvalg.setVisKameraer(!kartvalg.visKameraer);
        localStorageService.lagreILocalStorage(
            LocalStorageKey.LOCALSTORAGE_KART_VIS_KAMERAER,
            setLocalstorageDataMedGyldigDatoType(!kartvalg.visKameraer)
        );
    };

    const handleVisBrøytebilerToggle = () => {
        kartvalg.setVisBrøytebiler(!kartvalg.visBrøytebiler);
        localStorageService.lagreILocalStorage(
            LocalStorageKey.LOCALSTORAGE_KART_VIS_KJØRETØY,
            setLocalstorageDataMedGyldigDatoType(!kartvalg.visBrøytebiler)
        );
    };

    const handleVisLegendToggle = () => {
        kartvalg.settVisLegend !== undefined && kartvalg.settVisLegend(!kartvalg.visLegend);
        localStorageService.lagreILocalStorage(
            LocalStorageKey.LOCALSTORAGE_KART_VIS_LEGEND,
            setLocalstorageDataMedGyldigDatoType(!kartvalg.visLegend)
        );
    };

    const handleVisDarkmodeToggle = () => {
        kartvalg.settVisDarkmode(!kartvalg.visDarkmode);
        localStorageService.lagreILocalStorage(
            LocalStorageKey.LOCALSTORAGE_KART_VIS_DARKMODE,
            !kartvalg.visDarkmode
        );
    };

    return (
        <Container
            initial="collapsed"
            animate={åpen ? "open" : "closed"}
            exit="exit"
            variants={{
                open: { opacity: 1, x: 0 },
                collapsed: { opacity: 0, x: "100%" },
                exit: { opacity: 0, x: "100%" },
            }}
            transition={{
                duration: 0.3,
            }}
            ref={ref}
            id={id}
        >
            <Wrapper>
                <Tekst størrelse={"M"} noMargin={true} bold={true}>
                    Vis i kart
                </Tekst>
                <Checkbox
                    tabIndex={1}
                    key={"kamera"}
                    onChange={handleKameraerToggle}
                    checked={kartvalg.visKameraer}
                    value={"kamera"}
                    størrelse={"L"}
                    tekstStørrelse={"M"}
                >
                    Kamera og værstasjoner
                </Checkbox>
                <Checkbox
                    key={"brøytebiler"}
                    onChange={handleVisBrøytebilerToggle}
                    checked={kartvalg.visBrøytebiler}
                    value={"brøytebiler"}
                    størrelse={"L"}
                    tekstStørrelse={"M"}
                >
                    Kjøretøy
                </Checkbox>
                <Checkbox
                    key={"stengteveier"}
                    onChange={() =>
                        kartvalg.handleTrafikkmeldingToggle(!kartvalg.visTrafikkmeldinger)
                    }
                    checked={kartvalg.visTrafikkmeldinger}
                    value={"stengteveier"}
                    størrelse={"L"}
                    tekstStørrelse={"M"}
                    disabled={!kartvalg.trafikkmeldinger || !kartvalg.trafikkmeldinger.suksess}
                >
                    Trafikkmeldinger{" "}
                    {kartvalg.trafikkmeldinger === undefined && (
                        <Spinner storrelse="mindre" inline={true} />
                    )}
                </Checkbox>
                {kartvalg.visLegend !== undefined && (
                    <Checkbox
                        key={"vislegendintervaller"}
                        onChange={handleVisLegendToggle}
                        checked={kartvalg.visLegend}
                        value={"vislegendintervaller"}
                        størrelse={"L"}
                        tekstStørrelse={"M"}
                    >
                        Fargeforklaring
                    </Checkbox>
                )}
                <Checkbox
                    key={"visdarkmode"}
                    onChange={handleVisDarkmodeToggle}
                    checked={kartvalg.visDarkmode}
                    value={"visdarkmode"}
                    størrelse={"L"}
                    tekstStørrelse={"M"}
                >
                    Darkmode
                </Checkbox>
            </Wrapper>
        </Container>
    );
};

const Container = styled(motion.div)`
    position: fixed;
    background-color: var(--white);
    right: 70px;
    bottom: 126px;
    width: fit-content;
    height: fit-content;
    z-index: 1001;
    cursor: auto;
    box-shadow: 0px 0px 6px 2px rgba(53, 62, 67, 0.08);
    border: 2px solid var(--gray-dark);
`;

const Wrapper = styled.div`
    display: flex;
    flex-direction: column;
    padding: 20px;
    gap: 10px;
`;
