import { SisteKjøretøyDto } from "../../api/useHentSisteKjøretøy";
import { WebkameraDto } from "../../api/useHentWebkamera";
import React, { useContext, useEffect, useState } from "react";
import { KartdataContext } from "../../api/context/KartdataContext";
import { MapContainer, TileLayer } from "react-leaflet";
import styles from "./Kartvisning.module.css";
import { crsUtm33N } from "../utils/crs";
import { VeglenkePopupInfo } from "../popup/veglenke/VeglenkePopup";
import { KameraCluster } from "../marker/KameraCluster";
import { ChangeView } from "./ChangeView";
import { KjøretøyCluster } from "../marker/KjøretøyCluster";
import { ZoomKnapp } from "./ZoomKnapp";
import { RodeDto } from "../../api/useHentRoder";
import { getPopupType } from "../popup/popupUtils";
import { TrafikkmeldingerDto, useHentTrafikkmeldinger } from "../../api/useHentTrafikkmeldinger";
import {
    InformasjonsmeldingContext,
    informasjonsmeldinger,
} from "../../api/context/InformasjonsmeldingContext";
import { TrafikkmeldingClusterWrapper } from "./TrafikkmeldingClusterWrapper";
import { LatLngBoundsExpression } from "leaflet";
import { PilKnapp } from "../../components/fellesKomponenter/uttrekksmeny/PilKnapp";
import {
    KartParameterVelger,
    KombiKartvisningProps,
} from "../kartParameterVelger/KartParameterVelger";
import { RodeListeProps } from "../kartParameterVelger/RodeListe";
import { KartLagIkon } from "../../components/Ikon/KartLagIkon";
import { KartKnapp } from "../../components/fellesKomponenter/knapper/KartKnapp";
import { KartlagValg, KartvisningsValgProps } from "./KartlagValg";
import { Informasjonsmelding } from "../feil/Informasjonsmelding";
import { IntervallLegend } from "./IntervallLegend/IntervallLegend";
import localStorageService from "../../utils/storage/localstorageService";
import { LocalStorageKey } from "../../utils/storage/nøkler";
import { defaultStorageGyldigTilDato } from "../../utils/storage/typer";
import { isoDatoErIDag } from "../../utils/tid";
import { AnimatePresence } from "framer-motion";
import { useUntabableClusters } from "../../hooks/useUntabableClusters";
import { HoldClusterUtabbbar } from "./HoldClusterUtabbbar";
import { createGlobalStyle } from "styled-components";
import { useWindowSize } from "../../hooks/useWindowSize";
import { pilMenyWidth } from "../../utils/constants";
import { MenyÅpenContext } from "../../api/context/MenyÅpenContext";
import { useFirstRender } from "../../hooks/useFirstRender";

export enum InformasjonsPopupType {
    VEGLENKE,
    KJØRETØY,
    KAMERA,
    RODE,
    TRAFIKKMELDING,
    UKJENT,
}

export type PopupType =
    | VeglenkePopupInfo
    | WebkameraDto
    | SisteKjøretøyDto
    | RodeDto
    | TrafikkmeldingerDto;

export interface GenrellKartvisningProps {
    visKameraDefault: boolean;
    kjøretøydata: SisteKjøretøyDto[];
    kameradata: WebkameraDto[];
    kartBounds: LatLngBoundsExpression;
    children: React.ReactNode;
    popupInfo: PopupType | undefined;
    settPopupInfo: (popupInfo: PopupType | undefined) => void;
    rodeData: RodeListeProps;
    kombikartvalg: KombiKartvisningProps;
    visDarkmode: boolean;
    settVisDarkmode: (visDarkmode: boolean) => void;
}

export const KartvisningGenerell = ({
    visKameraDefault,
    kjøretøydata,
    kameradata,
    kartBounds,
    children,
    popupInfo,
    settPopupInfo,
    rodeData,
    kombikartvalg,
    visDarkmode,
    settVisDarkmode,
}: GenrellKartvisningProps) => {
    const { tidspunkt } = useContext(KartdataContext);
    const informasjonsmelding = useContext(InformasjonsmeldingContext);
    const { menyÅpen, oppdaterMenyÅpen } = useContext(MenyÅpenContext);
    const trafikkmeldinger = useHentTrafikkmeldinger(tidspunkt.verdi);
    const firstRender = useFirstRender();
    useUntabableClusters();

    const [visKameraer, setVisKameraer] = useState(visKameraDefault);
    const [visTrafikkmeldinger, setVisTrafikkmeldinger] = useState(false);
    const [visBrøytebiler, setVisBrøytebiler] = useState(true);
    const [visKartlagValg, setVisKartlagValg] = useState(false);
    const [visLegend, settVisLegend] = useState(true);

    const { windowWidth } = useWindowSize();
    const menywidth = windowWidth < 330 + pilMenyWidth ? windowWidth - pilMenyWidth : 330;

    useEffect(() => {
        const kameraerFraStorage = localStorageService.hentFraLocalStorage(
            LocalStorageKey.LOCALSTORAGE_KART_VIS_KAMERAER,
            defaultStorageGyldigTilDato
        );
        const kjøretøyFraStorage = localStorageService.hentFraLocalStorage(
            LocalStorageKey.LOCALSTORAGE_KART_VIS_KJØRETØY,
            defaultStorageGyldigTilDato
        );
        const legendFraStorage = localStorageService.hentFraLocalStorage(
            LocalStorageKey.LOCALSTORAGE_KART_VIS_LEGEND,
            defaultStorageGyldigTilDato
        );

        setVisTrafikkmeldinger(
            localStorageService.hentFraLocalStorage(
                LocalStorageKey.LOCALSTORAGE_KART_VIS_TRAFIKKMELDINGER,
                false
            )
        );
        setVisKameraer(
            isoDatoErIDag(kameraerFraStorage.gyldigDato) ? kameraerFraStorage.data : true
        );
        setVisBrøytebiler(
            isoDatoErIDag(kjøretøyFraStorage.gyldigDato) ? kjøretøyFraStorage.data : true
        );
        settVisLegend(isoDatoErIDag(legendFraStorage.gyldigDato) ? legendFraStorage.data : true);
        window.addEventListener("hashchange", åpneMenyPåFokusPåHovedinnhold);
        return () => window.removeEventListener("hashchange", åpneMenyPåFokusPåHovedinnhold);
    }, []);

    const handleTrafikkmeldingToggle = () => {
        if (!trafikkmeldinger) {
            informasjonsmelding.oppdater(informasjonsmeldinger.lasterStengteVeier);
        } else if (!trafikkmeldinger?.suksess || trafikkmeldinger.data.length === 0) {
            informasjonsmelding.oppdater(informasjonsmeldinger.ingenDataForStengteVeier);
        } else {
            setVisTrafikkmeldinger(!visTrafikkmeldinger);
            localStorageService.lagreILocalStorage(
                LocalStorageKey.LOCALSTORAGE_KART_VIS_TRAFIKKMELDINGER,
                !visTrafikkmeldinger
            );
        }
    };
    const åpneMenyPåFokusPåHovedinnhold = () => {
        if (location.hash === "#kartparametervelger") {
            oppdaterMenyÅpen(true);
        }
    };

    const kartvalg: KartvisningsValgProps = {
        visKameraer,
        setVisKameraer,
        visBrøytebiler,
        setVisBrøytebiler,
        visTrafikkmeldinger,
        handleTrafikkmeldingToggle,
        trafikkmeldinger,
        visLegend,
        settVisLegend,
        visDarkmode,
        settVisDarkmode,
    };

    return (
        <MapContainer
            className={styles.kartvisning}
            center={[64.6, 9.5022]}
            scrollWheelZoom
            crs={crsUtm33N}
            zoom={16}
            minZoom={4}
            maxZoom={16}
            zoomControl={false}
            keyboard={false}
        >
            <TileLayer
                attribution="© NVDB, Geovekst, kommunene og Open Street Map contributors (utenfor Norge)"
                url="https://services.geodataonline.no/arcgis/rest/services/Trafikkportalen/GeocacheTrafikkJPG/MapServer/tile/{z}/{y}/{x}"
                subdomains="123456789"
                className={visDarkmode ? "darkmodeKart" : ""}
                key={visDarkmode ? "darkmodeKart" : "lightmodeKart"}
            />

            {children}

            {kjøretøydata.length > 0 && (
                <KjøretøyCluster
                    kjøretøyListe={visBrøytebiler ? kjøretøydata : []}
                    settKjøretøyPopupInfo={settPopupInfo}
                    kjøretøyPopupInfo={
                        getPopupType(popupInfo) === InformasjonsPopupType.KJØRETØY
                            ? (popupInfo as SisteKjøretøyDto)
                            : undefined
                    }
                />
            )}

            {kameradata.length > 0 && (
                <KameraCluster
                    kameradata={visKameraer ? kameradata : []}
                    kameraPopupInfo={
                        getPopupType(popupInfo) === InformasjonsPopupType.KAMERA
                            ? (popupInfo as WebkameraDto)
                            : undefined
                    }
                    settKameraPopupInfo={settPopupInfo}
                />
            )}

            {trafikkmeldinger?.suksess && (
                <TrafikkmeldingClusterWrapper
                    visTrafikkmeldinger={visTrafikkmeldinger}
                    trafikkmeldinger={trafikkmeldinger}
                    popupInfo={popupInfo}
                    settPopupInfo={settPopupInfo}
                />
            )}
            <AnimatePresence>
                {menyÅpen && (
                    <KartParameterVelger
                        rodeData={rodeData}
                        kombikartvalg={kombikartvalg}
                        automatiskFokus={!firstRender}
                    />
                )}
            </AnimatePresence>
            <PilKnapp åpen={menyÅpen} onClick={() => oppdaterMenyÅpen(!menyÅpen)} />
            <MenyVariables width={menywidth} />

            <KartKnapp
                id={"karlagvalg"}
                right={20}
                bottom={126}
                onClick={() => setVisKartlagValg(!visKartlagValg)}
            >
                <KartLagIkon />
            </KartKnapp>

            <AnimatePresence>
                {visKartlagValg && (
                    <KartlagValg
                        åpen={visKartlagValg}
                        kartvalg={kartvalg}
                        onClose={() => setVisKartlagValg(false)}
                    />
                )}
            </AnimatePresence>
            <ZoomKnapp size="large" />
            {visLegend && <IntervallLegend />}
            <Informasjonsmelding />
            <ChangeView kartBounds={kartBounds} />
            <HoldClusterUtabbbar />
        </MapContainer>
    );
};

const MenyVariables = createGlobalStyle<{ width: number }>`
    :root {
        --meny-width: ${({ width }) => width}px;
    }
`;
