import { SisteKjøretøyDto } from "../../api/useHentSisteKjøretøy";
import { WebkameraDto } from "../../api/useHentWebkamera";
import React, { useEffect, useState } from "react";
import { VeglenkePopup, VeglenkePopupInfo } from "../popup/veglenke/VeglenkePopup";
import { Feature } from "geojson";
import { LeafletEvent } from "leaflet";
import { Produksjonslenke } from "../types/produksjonslenke";
import { RodeDto } from "../../api/useHentRoder";
import { getPopupType, mapTilRodePopupInfo, mapTilVeglenkePopupInfo } from "../popup/popupUtils";
import { finnKartBoundsForRoder } from "./kartBoundsUtils";
import { InformasjonsPopupType, KartvisningGenerell, PopupType } from "./KartvisningGenerell";
import { useTomProduksjonsvarsel } from "../../hooks/useTomProduksjonsvarsel";
import { RodePopup, RodePopupMedProduksjonInfo } from "../popup/roder/RodePopup";
import { Proj4GeoJSONFeature } from "proj4leaflet";
import { useSetFiltrerteLenker } from "./UseSetFiltrerteLenker";
import { RodeListeProps } from "../kartParameterVelger/RodeListe";
import { KombiKartvisningProps } from "../kartParameterVelger/KartParameterVelger";
import localStorageService from "../../utils/storage/localstorageService";
import { LocalStorageKey } from "../../utils/storage/nøkler";
import { tilAktivRodeGeojsonFeatures } from "./geoJson/geoJSONutilsRoder";
import { AnimatePresence } from "framer-motion";
import { GeoJsonRoder } from "./geoJson/GeoJsonRoder";
import { GeoJsonVeglenker } from "./geoJson/GeoJsonVeglenker";
import { defaultBounds, hentRodeIdFraVeglenke } from "../utils/kartUtils";

export interface ProduksjonskartVisningProps {
    produksjonslenker: Produksjonslenke[];
    kjøretøydata: SisteKjøretøyDto[];
    roder: RodeDto[];
    kameradata: WebkameraDto[];
}

export const ProduksjonskartVisning = ({
    produksjonslenker,
    kjøretøydata,
    kameradata,
    roder,
}: ProduksjonskartVisningProps) => {
    const [selektertRodeFraListe, settSelektertRodeFraListe] = useState<RodeDto | undefined>(
        undefined
    );
    const [aktivRodeFeature, settAktivRodeFeature] = useState<Proj4GeoJSONFeature | undefined>(
        undefined
    );
    const [aktivVeglenkeFeature, settAktivVeglenkeFeature] = useState<
        Proj4GeoJSONFeature | undefined
    >(undefined);
    const [popupInfo, settPopupInfo] = useState<PopupType | undefined>();
    const [filtrerteProduksjonslenker, settFiltrerteProduksjonslenker] = useState<
        Produksjonslenke[] | undefined
    >(undefined);
    const [visRoder, settVisRoder] = useState<boolean>(
        localStorageService.hentFraLocalStorage(LocalStorageKey.LOCALSTORAGE_KART_VIS_RODER, false)
    );
    const [visDarkmode, settVisDarkmode] = useState(false);
    const [visProduksjon, settVisProduksjon] = useState<boolean>(true);
    const [kartbounds, settKartbounds] = useState<L.LatLngBoundsExpression>(defaultBounds);

    useTomProduksjonsvarsel();
    useSetFiltrerteLenker({ produksjonslenker, settFiltrerteProduksjonslenker });

    const håndterSettVisRoder = (visRoder: boolean) => {
        settVisRoder(visRoder);
        localStorageService.lagreILocalStorage(
            LocalStorageKey.LOCALSTORAGE_KART_VIS_RODER,
            visRoder
        );
    };

    useEffect(() => {
        settKartbounds(finnKartBoundsForRoder(roder));
    }, [roder]);

    useEffect(() => {
        if (popupInfo && getPopupType(popupInfo) !== InformasjonsPopupType.VEGLENKE) {
            settAktivVeglenkeFeature(undefined);
        }
    }, [popupInfo]);

    useEffect(() => {
        if (selektertRodeFraListe === undefined) {
            settAktivRodeFeature(undefined);
        } else {
            settAktivRodeFeature(tilAktivRodeGeojsonFeatures(selektertRodeFraListe));
        }
    }, [selektertRodeFraListe]);

    useEffect(() => {
        const rodeIdPåVeglenke = hentRodeIdFraVeglenke(
            aktivVeglenkeFeature?.properties?.veglenkesekvensId,
            aktivVeglenkeFeature?.properties?.veglenkeNummer,
            roder
        );
        if (
            aktivRodeFeature?.id &&
            aktivVeglenkeFeature &&
            !rodeIdPåVeglenke.includes(aktivRodeFeature.id.toString())
        ) {
            settAktivRodeFeature(undefined);
        }
    }, [aktivVeglenkeFeature]);

    useEffect(() => {
        const visDarkmodeFraStorage = localStorageService.hentFraLocalStorage(
            LocalStorageKey.LOCALSTORAGE_KART_VIS_DARKMODE,
            false
        );
        settVisDarkmode(visDarkmodeFraStorage);
    }, []);

    const håndterVeglenkeKlikket = (event: LeafletEvent, feature: Feature) => {
        settPopupInfo(mapTilVeglenkePopupInfo(event, feature));
        settAktivVeglenkeFeature(feature);
    };

    const håndterRodeKlikket = (event: LeafletEvent, feature: Feature) => {
        settAktivRodeFeature(feature);
        settPopupInfo(mapTilRodePopupInfo(event, feature));
    };

    const rodeData: RodeListeProps = {
        roder,
        settPopupInfo: settPopupInfo,
        popupInfo: popupInfo,
        settSelektertRodeFraListe,
        visRoder,
        håndterSettVisRoder,
    };

    const kombikartvalg: KombiKartvisningProps = {
        visRoder,
        håndterSettVisRoder,
        visProduksjon,
        settVisProduksjon,
    };

    return (
        <KartvisningGenerell
            visKameraDefault={true}
            kjøretøydata={kjøretøydata}
            kameradata={kameradata}
            kartBounds={kartbounds}
            popupInfo={popupInfo}
            settPopupInfo={settPopupInfo}
            rodeData={rodeData}
            kombikartvalg={kombikartvalg}
            visDarkmode={visDarkmode}
            settVisDarkmode={settVisDarkmode}
        >
            <>
                {visRoder && (
                    <GeoJsonRoder
                        roder={roder}
                        aktivRodeFeature={aktivRodeFeature}
                        håndterRodeKlikket={håndterRodeKlikket}
                        visDarkmode={visDarkmode}
                    />
                )}

                {visProduksjon && (
                    <GeoJsonVeglenker
                        filtrerteProduksjonslenker={filtrerteProduksjonslenker}
                        håndterVeglenkeKlikket={håndterVeglenkeKlikket}
                        aktivVeglenkeFeature={aktivVeglenkeFeature}
                    />
                )}
                <AnimatePresence>
                    {getPopupType(popupInfo) === InformasjonsPopupType.RODE && (
                        <RodePopup
                            rodePopupInfo={popupInfo as RodePopupMedProduksjonInfo}
                            onClose={() => {
                                settPopupInfo(undefined);
                                settSelektertRodeFraListe(undefined);
                                settAktivRodeFeature(undefined);
                            }}
                        />
                    )}
                </AnimatePresence>
                <AnimatePresence>
                    {getPopupType(popupInfo) === InformasjonsPopupType.VEGLENKE && (
                        <VeglenkePopup
                            veglenkePopupInfo={popupInfo as VeglenkePopupInfo}
                            onClose={() => {
                                settPopupInfo(undefined);
                                settAktivVeglenkeFeature(undefined);
                            }}
                            produksjonslenker={produksjonslenker}
                        />
                    )}
                </AnimatePresence>
            </>
        </KartvisningGenerell>
    );
};
