import { useEffect, useRef, useState } from "react";
import { useMap } from "react-leaflet";
import styled from "styled-components";
import { useDisableClickPropagation } from "../../hooks/useDisableClickPropagation";

const enabledColor = "var(--gray-dark)";
const disabledColor = "var(--gray-dark-65)";

interface RelativeZoomProps {
    relative: boolean;
    parentWidth: number;
    parentHeight: number;
}

interface ZoomProps {
    size: "small" | "large";
    relativeProps?: RelativeZoomProps;
}

export const ZoomKnapp = ({ size, relativeProps }: ZoomProps) => {
    const map = useMap();

    const [maxZoomColor, setMaxZoomColor] = useState(enabledColor);
    const [minZoomColor, setMinZoomColor] = useState(enabledColor);

    const zoomButtonRef = useRef(null);

    useDisableClickPropagation(zoomButtonRef);

    useEffect(() => {
        map.removeEventListener("zoom");
        map.addEventListener(
            "zoom",
            () => {
                let zoom = map.getZoom();
                setMaxZoomColor(zoom >= map.getMaxZoom() ? disabledColor : enabledColor);
                setMinZoomColor(zoom <= map.getMinZoom() ? disabledColor : enabledColor);
            },
            { once: true }
        );
    }, []);

    const handleClick = (buttonId: string) => {
        if (buttonId === "zoomInn") {
            map.zoomIn();
        } else {
            map.zoomOut();
        }
    };

    return (
        <ZoomContainer ref={zoomButtonRef} relativeProps={relativeProps}>
            <ZoomPlus color={maxZoomColor} size={size} onClick={() => handleClick("zoomInn")}>
                <Plus color={maxZoomColor} />
            </ZoomPlus>
            <ZoomMinus color={minZoomColor} size={size} onClick={() => handleClick("zoomUt")}>
                <Minus color={minZoomColor} />
            </ZoomMinus>
        </ZoomContainer>
    );
};

interface SvgColor {
    color: string;
}

const Plus = ({ color }: SvgColor) => {
    return (
        <svg
            width="16"
            height="16"
            viewBox="0 0 16 16"
            fill="none"
            xmlns="http://www.w3.org/2000/svg"
        >
            <path
                d="M9.02486 0H6.94455V7.00574H0V8.99426H6.94455V16H9.02486V8.99426H16V7.00574H9.02486V0Z"
                fill={color}
            />
        </svg>
    );
};

const Minus = ({ color }: SvgColor) => {
    return (
        <svg
            width="16"
            height="2"
            viewBox="0 0 16 2"
            fill="none"
            xmlns="http://www.w3.org/2000/svg"
        >
            <rect width="16" height="2" fill={color} />
        </svg>
    );
};

interface ZoomPlusProps {
    size?: "small" | "large";
}

const ZoomPlus = styled.button<ZoomPlusProps>`
    display: flex;
    justify-content: center;
    align-items: center;
    width: ${(props) => (props.size == "small" ? "28px" : "44px")};
    height: ${(props) => (props.size == "small" ? "28px" : "44px")};
    background-color: ${({ color }) =>
        color == enabledColor ? "transparent" : "var(--gray-light)"};
    border: 0;
    border-color: ${({ color }) => color};
    outline: 0;
    position: relative;
    transition: all 0.1s ease-in-out;

    &:hover {
        box-shadow: ${({ color }) =>
            color == enabledColor &&
            `inset 0px 3px ${color},
            inset -3px 0px ${color},
            inset 0px -3px ${color},
            inset 3px 0px ${color}`};
        cursor: ${({ color }) => color == enabledColor && "pointer"};
        background-color: var(--gray-light);

        &:after {
            opacity: ${({ color }) => color == enabledColor && 0};
        }
    }

    &:focus {
        box-shadow: ${({ color }) =>
            color == enabledColor &&
            `inset 0 0 0 3px ${color}, inset 0 0 0 5px white,
            0 0 0 2px ${color}`};

        &:after {
            transition: all 0.1s ease-in-out;
            opacity: ${({ color }) => color == enabledColor && 0};
        }
    }

    box-shadow: ${({ color }) => `inset 0px 2px ${color}, inset -2px 0px ${color},
        inset 0px 0px transparent, inset 2px 0px ${color}`};

    &:after {
        content: "";
        position: absolute;
        height: 1px;
        left: 6px;
        right: 6px;
        background-color: ${({ color }) => color};
        bottom: -1px;
        opacity: 1;
    }
`;

const ZoomMinus = styled(ZoomPlus)`
    box-shadow: ${({ color }) => `inset 0px 0px ${color}, inset -2px 0px ${color},
        inset 0px -2px ${color}, inset 2px 0px ${color}`};

    &:after {
        display: none;
    }
`;

interface ZoomContainerProps {
    relativeProps?: RelativeZoomProps;
}

const ZoomContainer = styled.div.attrs<ZoomContainerProps>(({ relativeProps }) => ({
    style: {
        position: relativeProps?.relative ? "relative" : "fixed",
        left: relativeProps?.relative ? `${relativeProps.parentWidth - 32}px` : null,
        top: !relativeProps?.relative
            ? null
            : relativeProps.parentWidth > 510
            ? `${relativeProps.parentHeight - 78}px`
            : `${relativeProps.parentHeight - 94}px`,
        right: !relativeProps?.relative ? "20px" : null,
        bottom: !relativeProps?.relative ? "28px" : null,
    },
}))<ZoomContainerProps>`
    display: flex;
    flex-direction: column;
    background-color: white;
    box-shadow: 0px 0px 6px 2px rgba(53, 62, 67, 0.08);
    width: fit-content;
    z-index: 1001;

    &:hover {
        box-shadow: 0px 0px 8px 4px rgba(53, 62, 67, 0.3);
    }
`;
