import React, { useContext, useState } from "react";
import { DatoInput } from "./DatoInput";
import { ProduksjonsdataFilter } from "../filtre/filtervelger/ProduksjonsdataFilter";
import { TidspunktSisteImport } from "./TidspunktSisteImport";
import { MenyContainer } from "../../components/fellesKomponenter/uttrekksmeny/MenyContainer";
import { RodeListe, RodeListeProps } from "./RodeListe";
import { KartvisningsValg } from "./KartvisningsValg";
import { ProduksjonslenkerContext } from "../../api/context/ProduksjonslenkerContext";
import { EksporterAllData } from "./EksporterAllData";
import styled from "styled-components";
import { DataInput } from "./DataInput";
import { erGyldigNumeriskInput } from "../../utils/inputValidering";
import { ØktfilterContext } from "../../api/context/ØktContext";
import { HastighetsfilterContext } from "../../api/context/HastighetsfilterContext";
import tekster from "../../tekster.json";
import { EksternLenke } from "../../components/fellesKomponenter/lenker/EksternLenke";
import { sideInfo } from "../../sider";

interface KartParameterVelgerProps {
    rodeData: RodeListeProps;
    kombikartvalg: KombiKartvisningProps;
    automatiskFokus: boolean;
}

export interface KombiKartvisningProps {
    visRoder: boolean;
    håndterSettVisRoder: (visRoder: boolean) => void;
    visProduksjon: boolean;
    settVisProduksjon: (visProduksjon: boolean) => void;
}

export const KartParameterVelger = ({
    rodeData,
    kombikartvalg,
    automatiskFokus,
}: KartParameterVelgerProps) => {
    const [øktError, setØktError] = useState<boolean>(false);
    const [hastighetError, setHastighetError] = useState<boolean>(false);

    const { produksjonslenker } = useContext(ProduksjonslenkerContext);
    const øktfilter = useContext(ØktfilterContext);
    const hastighetsfilter = useContext(HastighetsfilterContext);

    const handleSubmitØkt = (økt: string) => {
        const gyldigInput = erGyldigNumeriskInput(økt);
        setØktError(!gyldigInput);
        if (gyldigInput) {
            øktfilter.oppdaterFilter({ verdi: økt.replaceAll(" ", ""), valgt: true });
        }
    };

    const handleResetØkt = () => {
        setØktError(false);
        øktfilter.oppdaterFilter({ verdi: "", valgt: false });
    };

    const handleSubmitHastighet = (hastighet: string) => {
        const gyldigInput = erGyldigNumeriskInput(hastighet);
        setHastighetError(!gyldigInput);
        if (gyldigInput) {
            hastighetsfilter.oppdaterFilter({ verdi: hastighet.replaceAll(" ", ""), valgt: true });
        }
    };

    const handleResetHastighet = () => {
        setHastighetError(false);
        hastighetsfilter.oppdaterFilter({ verdi: "", valgt: false });
    };

    return (
        <section id={"kartparametervelger"}>
            <MenyContainer>
                <MenyInnholdContainer>
                    <KartvisningsValg
                        kombikartvalg={kombikartvalg}
                        automatiskFokus={automatiskFokus}
                    />
                </MenyInnholdContainer>

                <MenyInnholdContainer>
                    <TidspunktSisteImport />
                    <DatoInput />
                </MenyInnholdContainer>

                <MenyInnholdContainer>
                    <ProduksjonsdataFilter />
                    <DataInput
                        tittel={"Minimum snittfart/minutt"}
                        handleSubmit={handleSubmitHastighet}
                        resetData={handleResetHastighet}
                        errormelding={["Skriv inn et tall"]}
                        error={hastighetError}
                        setError={setHastighetError}
                        hjelpetekst={tekster.minimumSnittfart}
                    />
                    <DataInput
                        tittel={"Velg økt"}
                        handleSubmit={handleSubmitØkt}
                        resetData={handleResetØkt}
                        errormelding={["Skriv inn et tall"]}
                        error={øktError}
                        setError={setØktError}
                        hjelpetekst={tekster.velgØkt}
                        laster={øktfilter.laster}
                    />
                    <EksternLenke href={sideInfo.øktdetaljer.href} størrelse={"M"}>
                        Se øktdetaljer
                    </EksternLenke>
                    <EksporterAllData produksjonslenker={produksjonslenker} />
                </MenyInnholdContainer>

                <MenyInnholdContainer>
                    <RodeListe
                        roder={rodeData.roder}
                        settPopupInfo={rodeData.settPopupInfo}
                        popupInfo={rodeData.popupInfo}
                        settSelektertRodeFraListe={rodeData.settSelektertRodeFraListe}
                        visRoder={kombikartvalg.visRoder}
                        håndterSettVisRoder={kombikartvalg.håndterSettVisRoder}
                    />
                </MenyInnholdContainer>
            </MenyContainer>
        </section>
    );
};

export const MenyInnholdContainer = styled.div`
    display: flex;
    flex-direction: column;
    gap: 0.7rem;
`;
