import React, { useContext, useEffect, useState } from "react";
import { InformasjonPopup } from "../InformasjonPopup";
import { ApiRespons } from "../../../api/api";
import { Filter, ProduksjonsfilterContext } from "../../../api/context/ProduksjonsfilterContext";
import {
    erFilterGjeldene,
    produksjonslenkeTilFilterKriterie,
} from "../../filtre/filtrerProduksjon/filtrerProduksjon";
import { KartdataContext } from "../../../api/context/KartdataContext";
import { loggFrontend, LoggNivå } from "../../../utils/loggFrontend";
import { ProduksjonsdetaljerCacheContext } from "../../../api/context/ProduksjonsdetaljerCacheContext";
import { PopupInnhold } from "./PopupInnhold";
import { Produksjonslenke } from "../../types/produksjonslenke";
import { Produksjonsdetaljer } from "../../types/produksjonsdetaljer";
import {
    Hastighetsfilter,
    HastighetsfilterContext,
} from "../../../api/context/HastighetsfilterContext";
import { useOktContext, Øktfilter, ØktfilterContext } from "../../../api/context/ØktContext";
import { RoderContext } from "../../../api/context/RodeContext";
import { Tidsfilter, TidsfilterContext } from "../../../api/context/TidsfilterContext";
import { hentRodeFraVeglenke } from "../../utils/kartUtils";
import { veglenkePopupUndertekst } from "./VeglenkePopupUndertekst";

export interface VeglenkePopupInfo {
    kartlinje: any;
    veglenkesekvensId: number;
    veglenkeNummer: number;
    timerOgMinutterSiden?: string;
}

interface VeglenkePopupProps {
    veglenkePopupInfo: VeglenkePopupInfo;
    onClose: () => void;
    produksjonslenker: Produksjonslenke[];
}

export const VeglenkePopup = ({
    veglenkePopupInfo,
    onClose,
    produksjonslenker,
}: VeglenkePopupProps) => {
    const [produksjonsdetaljer, setProduksjonsdetaljer] =
        useState<ApiRespons<Produksjonsdetaljer[]>>();
    const { produksjonsfilter } = useContext(ProduksjonsfilterContext);
    const { hastighetsfilter } = useContext(HastighetsfilterContext);
    const { øktfilter } = useContext(ØktfilterContext);
    const { kontrakt, tidspunkt } = useContext(KartdataContext);
    const { roder } = useContext(RoderContext);
    const { tidsfilter } = useContext(TidsfilterContext);
    const { gjeldendeØkt } = useOktContext();
    const { pushToCache, hentProduksjonsdetaljer } = useContext(ProduksjonsdetaljerCacheContext);

    const hentMålepunkter = (veglenkeNummer: number, veglenkesekvensId: number) => {
        const punkter = filterProduksjonslenker(
            veglenkeNummer,
            veglenkesekvensId,
            produksjonsfilter,
            hastighetsfilter,
            øktfilter,
            tidsfilter,
            [...produksjonslenker, ...gjeldendeØkt]
        ).flatMap((produksjonslenke) => produksjonslenke.målepunkter);

        if (punkter.length === 0) {
            const valgteFiltere = produksjonsfilter
                .filter((f) => f.valgt)
                .map((f) => f.navn)
                .toString();
            const loggInfo = `Ingen produksjonsdetaljer for veglenke ${veglenkeSekvensOgLenkeNummer}\n kontrakt: ${
                kontrakt.aktiv?.navn
            },\n tidspunkt: ${tidspunkt.verdi.toISOString()},\n valgteFiltere:${valgteFiltere}`;
            loggFrontend({
                loggNivå: LoggNivå.Warn,
                customInformasjon: loggInfo,
            });
        }
        return punkter;
    };

    useEffect(() => {
        const hentData = async () => {
            const målepunkter = hentMålepunkter(
                veglenkePopupInfo.veglenkeNummer,
                veglenkePopupInfo.veglenkesekvensId
            );
            setProduksjonsdetaljer(undefined);
            const data = await hentProduksjonsdetaljer(målepunkter, tidspunkt.verdi);
            if (data != undefined) {
                setProduksjonsdetaljer(data);
                pushToCache(målepunkter.toString(), data);
            }
        };
        hentData();
    }, [veglenkePopupInfo]);

    const veglenkeSekvensOgLenkeNummer = `${veglenkePopupInfo.veglenkesekvensId} - ${veglenkePopupInfo.veglenkeNummer}`;
    const roderPåVeglenken = hentRodeFraVeglenke(veglenkeSekvensOgLenkeNummer, roder?.data);

    return (
        <InformasjonPopup
            tittel="Målepunkt for veglenke"
            undertekstHtml={veglenkePopupUndertekst(roderPåVeglenken, veglenkeSekvensOgLenkeNummer)}
            ikonPath={popupIkon(produksjonsdetaljer)}
            detaljWidth={680}
            skalHaDetaljMinHeight={true}
            onClose={onClose}
        >
            <PopupInnhold
                produksjonsdetaljer={produksjonsdetaljer}
                roderPåVeglenken={roderPåVeglenken}
                veglenkeSekvensOgLenkeNummer={veglenkeSekvensOgLenkeNummer}
                timerOgMinutterSiden={veglenkePopupInfo.timerOgMinutterSiden}
            />
        </InformasjonPopup>
    );
};

export const filterProduksjonslenker = (
    veglenkeNummer: number,
    veglenkesekvensId: number,
    produksjonsfilter: Filter[],
    hastighetsfilter: Hastighetsfilter,
    øktfilter: Øktfilter,
    tidsfilter: Tidsfilter[],
    produksjonslenker: Produksjonslenke[]
): Produksjonslenke[] =>
    produksjonslenker.filter(
        (produksjonslenke) =>
            produksjonslenke.veglenkeNummer === veglenkeNummer &&
            produksjonslenke.veglenkesekvensId === veglenkesekvensId &&
            erFilterGjeldene(
                produksjonsfilter,
                hastighetsfilter,
                øktfilter,
                produksjonslenkeTilFilterKriterie(produksjonslenke),
                tidsfilter,
                produksjonslenke.minutterSiden,
                produksjonslenke.hastighetKmH,
                produksjonslenke.økt
            )
    );

const popupIkon = (produksjonsdetaljer?: ApiRespons<Produksjonsdetaljer[]>) =>
    produksjonsdetaljer?.suksess === false
        ? "/ikon/feil-kryss.svg"
        : "/ikon/kart/broytebil-sirkel.svg";
