import {
    CartesianGrid,
    Label,
    Legend,
    Line,
    LineChart,
    ReferenceArea,
    ReferenceLine,
    ResponsiveContainer,
    Tooltip,
    XAxis,
    YAxis,
} from "recharts";
import {
    formatAxisTid,
    hentGraftittel,
    hentReflinjeBenevning,
    hentStartOgSluttTid,
    hentTicksXaxis,
    hentYAxisLabel,
} from "./grafUtils";
import { CustomTooltip } from "./CustomTooltip";
import React, { ReactNode, useState } from "react";
import { CustomLegend } from "./CustomLegend";
import { Grafdata } from "./Grafer";
import { HastighetsType } from "./HastighetsGraf";
import { DoseringsType } from "./DoseringsGraf";
import dayjs from "dayjs";
import { GeneriskGrafvalg } from "./GeneriskGrafvalg";
import styled from "styled-components";
import { breakpointNettbrett } from "../../utils/constants";

export enum GrafType {
    hastighet = "hastighet",
    dosering = "dosering",
}

interface Props {
    type: GrafType;
    dataKey: HastighetsType | DoseringsType;
    tickArrayYaxis: number[];
    grafdata: Grafdata[];
    setAktivHoverRecord: (record: number[] | undefined) => void;
    reflinjeIKart?: number;
    children: ReactNode;
}
export const GeneriskGraf = ({
    type,
    dataKey,
    tickArrayYaxis,
    grafdata,
    setAktivHoverRecord,
    reflinjeIKart,
    children,
}: Props) => {
    const { startTid, sluttTid } = hentStartOgSluttTid(grafdata);
    const defaultXaxisDomain = [startTid.valueOf(), sluttTid.valueOf()];
    const defaultXaxisTicks = hentTicksXaxis(startTid, sluttTid);
    const [xaxisDomain, setXaxisDomain] = useState<number[]>(defaultXaxisDomain);
    const [xaxisTicks, setXaxisTicks] = useState<number[]>(defaultXaxisTicks);
    const [seleksjonstidStart, setSeleksjonstidStart] = useState<number | undefined>(undefined);
    const [seleksjonstidSlutt, setSeleksjonstidSlutt] = useState<number | undefined>(undefined);
    const [synligGrafdata, setSynligGrafdata] = useState<Grafdata[]>(grafdata);
    const [visTilbakestillknapp, setVisTilbakestillknapp] = useState(false);
    const handleSeleksjon = () => {
        if (
            seleksjonstidStart === seleksjonstidSlutt ||
            seleksjonstidSlutt === undefined ||
            seleksjonstidStart === undefined
        ) {
            setSeleksjonstidStart(undefined);
            setSeleksjonstidSlutt(undefined);
            setVisTilbakestillknapp(false);
            return;
        }

        const domain = [seleksjonstidStart, seleksjonstidSlutt].sort();
        setXaxisDomain(domain);
        setXaxisTicks(hentTicksXaxis(dayjs(domain[0]), dayjs(domain[1])));
        setSeleksjonstidSlutt(undefined);
        setSeleksjonstidStart(undefined);
        setVisTilbakestillknapp(true);
    };

    const handleReset = () => {
        setXaxisDomain(defaultXaxisDomain);
        setXaxisTicks(defaultXaxisTicks);
        setVisTilbakestillknapp(false);
    };

    const renderCustomizedLegend = () => {
        return (
            <CustomLegend
                grafdata={synligGrafdata}
                setGrafdata={(data: Grafdata[]) => setSynligGrafdata(data)}
            />
        );
    };

    const yAxisLabel = hentYAxisLabel(type, dataKey, grafdata);
    const referanselinjeBenvevning = hentReflinjeBenevning(type, dataKey, grafdata);
    const tittel = hentGraftittel(type, dataKey);

    return (
        <div>
            <GrafContainer>
                <ResponsiveContainer height={360} minWidth={550}>
                    <LineChart
                        margin={{ top: 50, right: 40, left: 20, bottom: 30 }}
                        style={{ userSelect: "none" }}
                        onMouseDown={(e) =>
                            e !== null && setSeleksjonstidStart(Number(e.activeLabel))
                        }
                        onMouseMove={(e) =>
                            seleksjonstidStart &&
                            e !== null &&
                            setSeleksjonstidSlutt(Number(e.activeLabel))
                        }
                        onMouseUp={handleSeleksjon}
                    >
                        <text
                            x="50%"
                            y={10}
                            fill="black"
                            textAnchor="middle"
                            dominantBaseline="central"
                        >
                            <tspan fontSize="20">{tittel}</tspan>
                        </text>
                        {synligGrafdata
                            ?.filter((data) => data.visible)
                            .map((data, index) => (
                                <Line
                                    key={"linje" + index}
                                    data={data.data}
                                    type="monotone"
                                    dataKey={dataKey}
                                    stroke={data.color}
                                    strokeWidth={2}
                                    dot={false}
                                    activeDot={{ r: 6 }}
                                    name={data.value}
                                    isAnimationActive={false}
                                />
                            ))}

                        <CartesianGrid strokeDasharray="3 3" />
                        <XAxis
                            allowDataOverflow
                            dataKey="numeriskTid"
                            type="number"
                            tickFormatter={formatAxisTid}
                            allowDuplicatedCategory={false}
                            ticks={xaxisTicks}
                            domain={xaxisDomain}
                            tickCount={20}
                            user-select={"none"}
                        >
                            <Label value="Klokkeslett" offset={-15} position="insideBottom" />
                        </XAxis>
                        <YAxis domain={[0, Math.max(...tickArrayYaxis)]} ticks={tickArrayYaxis}>
                            <Label
                                value={yAxisLabel}
                                offset={15}
                                position="insideBottomLeft"
                                angle={-90}
                            />
                        </YAxis>
                        <Legend
                            wrapperStyle={{ top: 25 }}
                            verticalAlign="top"
                            align="center"
                            height={30}
                            content={renderCustomizedLegend}
                        />
                        <Tooltip
                            content={
                                <CustomTooltip
                                    setAktivHoverRecord={setAktivHoverRecord}
                                    type={type}
                                />
                            }
                        />

                        {seleksjonstidStart && seleksjonstidSlutt ? (
                            <ReferenceArea
                                x1={seleksjonstidStart}
                                x2={seleksjonstidSlutt}
                                strokeOpacity={0.3}
                            />
                        ) : null}
                        {reflinjeIKart && (
                            <ReferenceLine
                                y={reflinjeIKart}
                                label={{
                                    value: reflinjeIKart + referanselinjeBenvevning,
                                    position: "top",
                                    fontSize: 16,
                                    offset: 10,
                                    fontWeight: "bold",
                                    fill: "red",
                                }}
                                stroke="red"
                                strokeDasharray="3 3"
                                strokeWidth={3}
                            />
                        )}
                        <CartesianGrid strokeDasharray="3 3" />
                    </LineChart>
                </ResponsiveContainer>
            </GrafContainer>
            <GeneriskGrafvalg visTilbakestillknapp={visTilbakestillknapp} handleReset={handleReset}>
                {children}
            </GeneriskGrafvalg>
        </div>
    );
};

const GrafContainer = styled.div`
    @media (max-width: ${breakpointNettbrett}) {
        overflow-x: scroll;
        overflow-y: hidden;
    }
`;
